
#define BAUD_DEF    0		/* use default				*/
#define BAUD_MAX    1		/* use highest possible speed		*/
#define BAUD_FOUND  2		/* leave where reset occurred		*/

#define M_SUCCEED   0
#define M_FAIL	    1
#define MAXREPLY    20
#define QBUFSIZ     30

/* modem identifier codes   */
#define ID_UNKNOWN  0
#define ID_300	    1
#define ID_12	    2
#define ID_12PLUS   3
#define ID_24	    4
#define ID_12CLONE  5
#define ID_24CLONE  6

/* manifest constants for use with modem functions  */
#define M_OK	    0		/* successful command			*/
#define M_CONN3     1		/* carrier at 300 baud			*/
#define M_RING	    2		/* call incoming			*/
#define M_NODCD	    3		/* no carrier detected			*/
#define M_CMDERR    4		/* error in smartmodem command		*/
#define M_CONN12    5		/* carrier at 1200 baud 		*/
#define M_NOTONE    6		/* no dial tone heard			*/
#define M_BUSY	    7		/* busy signal heard			*/
#define M_NOANSW    8		/* sillence not heard after @		*/
#define M_RESERV    9		/* ** don't use this!                   */
#define M_CONN24    10		/* carrier at 2400 baud 		*/
#define M_NORING    11		/* no ring detected			*/
#define M_BADMSG    12		/* invalid smartmodem response		*/
#define M_ONLINE    13		/* can't dial into a high dcd           */
#define M_NILNUM    14		/* blank dial string			*/
#define M_USRCAN    15		/* call canceled from keyboard		*/
#define M_SIOERR    16		/* SIO will not clear			*/


/*
STRUCT DEF:	modem
FILE:		MODEM.H
DESCRIPTION:	Support for a smart modem.
*/

#define CMDBUFSIZ   41	    /* capacity of Smartmodem's command buffer  */
#define CMDSIZ	    8	    /* max storage needed for Smartmodem command*/

struct modem
{
    unsigned	ibdelay;	/*  0 interbyte delay			*/
    BOOL    cmdechoflg; 	/*  1 modem commands to console 	*/
    char    csi[CMDSIZ];	/*  2 command sequence introducer	*/
    char    eocstr[CMDSIZ];	/*  3 end-of-command character(s)	*/
    char    quietcmd[CMDSIZ];	/*  4 Q cmd: send result codes		*/
    char    cmdbuff[CMDBUFSIZ]; /*  5 buile all commands here		*/
    int     cmdt_out;		/*  6 Tsecs to wait after a modem command   */
    int     cmdretry;		/*  7 number of times to try a command	*/
    unsigned	guardtime;	/*  8 escape guard time 		*/
    BOOL    iscard;		/*  9 is this a board modem?		*/
    int     baudtry[6]; 	/* 10 baud rates to try during init	*/
    char    escape[CMDSIZ];	/* 11 (S2) online escape byte = '|'	*/
    unsigned	baudfound;	/* 12 baud rate which modem found	*/
    BOOL    rs232flg;		/* 13 monitor RS-232 signals?		*/
    short   smid;		/* 14 result of modem's product ID cmd  */
    short   romck;		/* 15 rom checksum			*/
    short   modtype;		/* 16 our code for modem		*/
    BOOL    configokflg;	/* 17 modem successfully configured	*/
    BOOL    baudmode;		/* 18 baud rate initialization mode	*/
    unsigned	baudmax;	/* 19 maximum baud rate of modem	*/
    char    pri_config[CMDBUFSIZ];  /* 20 main configuration string	*/
    char    dialmode[CMDSIZ];	/* 21 tone or pulse dialling		*/
    char    echocmd[CMDSIZ];	/* 22 E cmd: echoes in command mode	*/
    char    verbosecmd[CMDSIZ]; /* 23 V cmd: word or digit result codes */
    char    autoans[CMDSIZ];	/* 24 (S0)# of rings before autoanser	*/
    char    dtwait[CMDSIZ];	/* 25 (S6)dialtone wait (secs)		*/
    char    dcdwait[CMDSIZ];	/* 26 (S7)dcd timeout wait (secs)	*/
    char    speaker[CMDSIZ];	/* 27 speaker mode			*/
    char    sec_config[CMDBUFSIZ];  /* 28 config str for evolutionary cmds  */
    char    xcmd[CMDSIZ];	/* 29 result code set			*/
    char    okalpha[12];	/* 30 alpha chars ok in dial string	*/
    char    speakvol[CMDSIZ];	/* 31 speaker volume			*/
    char    firmdcd[CMDSIZ];	/* 32 2400's DCD firmware config switch */
    char    firmdtr[CMDSIZ];	/* 33 2400's DTR firmware config switch */
    char    unconfig[CMDBUFSIZ];    /* 34 string to send to smodem at exit  */
    char    dialbuff[CMDBUFSIZ];    /* 35 build numeric dial string here    */
    char    dialcmd[CMDSIZ];	/* 36 dial command			*/
    char    enddial[CMDSIZ];	/* 37 dial suffix--e.g., ';' or 'R'	*/
    char    anscmd[CMDSIZ];	/* 38 answer command			*/
    BOOL    autobrflg;		/* 39 switch baud rate to incoming call */
    char    hupcmd[CMDSIZ];	/* 40 hang up command			*/
};
