
#include <stdio.h>
#include "siodef.h"
#include "u8250.h"
#include "sioctl.h"
#include "ascii.h"
#include "modem.h"

#define COMM1	(BYTE *)0x3F8	    /* IBM PC's first serial port       */
#define COMM2	(BYTE *)0x2F8	    /* IBM PC's second serial port      */
#define COMM_SL4	(BYTE *)0x2B8
#define DP_OFF	0		    /* offset of UART data port 	*/
#define SP_OFF	5		    /* offset of UART status port	*/

#define MAX_PORTS   16

BYTE	inport(), s_rcv(), s_rcvstat(), s_xmitstat();
void	outport(), s_xmit();
unsigned peekw();
BYTE	s_istat(), s_iread();

/*
ARRAY NAME:	v_regA
TYPE:		Two-dimensional array of BYTE.
LEVEL:		0
FILE:		IBMPC.C
DESCRIPTION:	Used for virtual UART registers.
*/

BYTE v_regA[5][2] =
{
/* VIR	USR */
    0,	0,		    /* 0 = baud low byte	*/
    0,	0,		    /* 1 = baud high byte	*/
    0,	0,		    /* 2 = interrupt enable	*/
    0,	0,		    /* 3 = data format		*/
    0,	0,		    /* 4 = RS232 output control */
};

/*
STRUCT NAME:	parityA, stopsA, dlenA
TYPE:		struct vregbits_
LEVEL:		0
FILE:		IBMPC.C
DESCRIPTION:	SIO control structures for parity, stop bits, and data length.
*/

struct vregbits_ parityA =  /* for 8250 UART	*/
{
    PARITYMASK, 	    /* AND mask to apply to this register	*/
    {NONE_MASK, ODD_MASK, EVEN_MASK, MARK_MASK, SPACE_MASK}, /* OR masks*/
    VFMT,		    /* virtual register -- rank in v_rega	*/
    3,			    /* offset of physical register from uart_base*/
    SUPPLIED,		    /* rank in mask table of current setting	*/
    NONE,		    /* rank to use in initialization		*/
    NIL 		    /* no more structures for parity		*/
};

struct vregbits_ stopsA =   /* for 8250 UART	*/
{
    STOPMASK,
    {ONE_MASK, ONE_5_MASK, TWO_MASK, -1, -1},
    VFMT,
    3,
    SUPPLIED,
    STOP1,
    NIL 		  /* no more structures for stop bits	      */
};

struct vregbits_ dlenA =    /* for 8250 UART	*/
{
    DLMASK,
    {DL5_MASK, DL6_MASK, DL7_MASK, DL8_MASK, -1},
    VFMT,
    3,
    SUPPLIED,
    DL8,
    NIL 		    /* no more steuctures for data length	*/
};

/*
STRUCT NAME:	baudA
TYPE:		struct vbaud_
LEVEL:		0
FILE:		IBMPC.C
DESCRIPTION:	SIO control structure for baud rate.
*/

struct vbaud_ baudA =
{
    VBAUDLO,		    /* virtual register for lo byte		*/
    VBAUDHI,		    /* virtual register for hi byte		*/
    0,			    /* offset of UART lsb baud reg		*/
    1,			    /* offset of UART msb baud reg		*/
    {
    {BH_50,	BL_50	}, {BH_75,	BL_75	}, {BH_110,	BL_110	},
    {BH_134,	BL_134	}, {BH_150,	BL_150	}, {BH_300,	BL_300	},
    {BH_600,	BL_600	}, {BH_1200,	BL_1200	}, {BH_1800,	BL_1800	},
    {BH_2000,	BL_2000	}, {BH_2400,	BL_2400	}, {BH_3600,	BL_3600	},
    {BH_4800,	BL_4800	}, {BH_7200,	BL_7200	}, {BH_9600,	BL_9600	},
    {BH_19K2,	BL_19K2 }, {BH_38K4,	BL_38K4 }
    },
    SUPPLIED,		    /* rank of current baud in divisor table	*/
    BAUD1200		    /* start at 1200 baud			*/
};

/*
STRUCT NAME:	pin4A, pin20A, gp1outA, gp2outA, breakbA
TYPE:		vout232_
LEVEL:		0
FILE:		IBMPC.C
DESCRIPTION:	Declaracrion and initialization of RS-232 and BREAK output
		control structures.
*/

struct vout232_ pin4A = 	/*******    RTS	       ******/
{
    RTSMASK,			/* mask to reset RTS	    */
    V232OUT,			/* virtual reg number	    */
    4,				/* offset of UART reg	    */
    SUPPLIED,			/* state of RTS bit now     */
    OFF,			/* state for starup	    */
    NIL 			/* pointer to next	    */
};

struct vout232_ pin20A =	/******     DTR        ******/
{
    DTRMASK,
    V232OUT,
    4,
    SUPPLIED,
    OFF,
    NIL
};

struct vout232_ gp1outA =	/******     GPO1       ******/
{
    GPO1MASK,
    V232OUT,
    4,
    SUPPLIED,
    OFF,
    NIL
};

struct vout232_ gp2outA =	/******     GPO2       ******/
{
    GPO2MASK,
    V232OUT,
    4,
    SUPPLIED,
    OFF,
    NIL
};

struct vout232_ breakbA =	/******  BREAK signal  ******/
{
    BRK_MASK,
    VFMT,
    3,
    SUPPLIED,
    OFF,
    NIL
};

/*
STRUCT NAMES:	pin5A, pin6A, pin8A, pin22A
TYPE:		vin232_
LEVEL:		0
FILE:		IBMPC.C
DESCRIPTION:	vin232_ structures for RS-232 control
*/
BYTE _vstat232();

struct vin232_ pin5A =		/****** 	   CTS		   ******/
{
    6,				/* offset of UART register		*/
    CTSMASK,			/* AND mask for this bit		*/
    SUPPLIED,			/* value of bit at last test		*/
    _vstat232			/* pointer to level-1 function		*/
};

struct vin232_ pin6A =		/****** 	   DSR		   ******/
{
    6,
    DSRMASK,
    SUPPLIED,
    _vstat232
};

struct vin232_ pin8A =		/****** 	   DCD		   ******/
{
    6,
    DCDMASK,
    SUPPLIED,
    _vstat232
};

struct vin232_ pin22A = 	/****** 	    RI		   ******/
{
    6,
    RIMASK,
    SUPPLIED,
    _vstat232
};

/*
STRUCT NAME:	octlA
TYPE:		struct soctl_
FILE:		IBMPC.C
DESCRIPTION:	Declares and initializes an soctl_ output control structure.
COMMENTS:	The numbering in the comments corresponds to that in the
		soctl_ structure definition.
*/

struct soctl_ octlA =
{
    OFF,		    /*	0 "raw" output flag			*/
    NOXLAT,		    /*	1 end-of-line mode			*/
    OFF,		    /*	2 console output flag			*/
    OFF,		    /*	3 echo-wait flag			*/
    _1_SEC_0 * 5,	    /*	4 echo-wait befoer timeout in Tsecs	*/
    UP_LOW,		    /*	5 upper/lower case conversion mode	*/
    XON_ONLY,		    /*	6 flow control mode			*/
    DC3,		    /*	7 XOFF character			*/
    DC1,		    /*	8 XON character if mode >= 0		*/
    ON, 		    /*	9 strip high bits flag			*/
    ON, 		    /* 10 remove unwanted controls flag 	*/
    {NUL, BEL, BS, HT, LF, CR, FF, DC3, DC1, DOS_EOF},	/* 11 permitted controls    */
    10, 		    /* 12 number of controls in above list	*/
    OFF,		    /* 13 translation/removal flag		*/
    {NUL},			    /* 14 translation/removal list		*/
    0,			    /* 15 no entries in above array		*/
    OFF,		    /* 16 no interbyte delay flag		*/
    _0_SEC_05,		    /* 17 delay in Tsecs			*/
    ON, 		    /* 18 end-of-line pause flag		*/
    LF, 		    /* 19 character to puase after		*/
    _0_SEC_4,		    /* 20 pause in Tsecs			*/
    ESC,		    /* 21 keyboard abort character		*/
    SUPPLIED,		    /* 22 'last character' buffer		*/
    SUPPLIED		    /* 23 flag to flush last character		*/
};

/*
STRUCT DEF:	ictlA
TYPE:		struct sictl_
FILE:		IBMPC.C
DESCRIPTION:	Declares and initializes an input control structure.
COMMENTS:	The numbering in the comments corresponds to that in the
		sictl_ structure definition.
*/

struct sictl_ ictlA =
{
    OFF,		    /*	0 timeout-on-input flag 		*/
    _0_SEC_05,		    /*	1 default timeout in Tsecs		*/
    OFF,		    /*	2 "rew" input flag			*/
    OFF,		    /*	3 echo input back to sender flag	*/
    ON, 		    /*	4 reset high bit flag			*/
    NOXLAT,		    /*	5 end-of-line mode			*/
    OFF,		    /*	6 console output flag			*/
    UP_LOW,		    /*	7 upper/lower case conversion mode	*/
    DC3,		    /*	8 XOFF (pause) character to send	*/
    DC1,		    /*	9 XON (release) character to send	*/
    ON, 		    /* 10 remove unwanted controls flag 	*/
    {NUL, BEL, BS, HT, LF, CR, FF, DC3, DC1, DOS_EOF},	/* 11 permitted controls    */
    10, 		    /* 12 number of controls in above list	*/
    OFF,		    /* 13 byte translation/removal mode 	*/
    {NUL},		    /* 14 translation/removal array		*/
    0,			    /* 15 no entries in above array		*/
    SUPPLIED,		    /* 16 'last character' buffer		*/
    SUPPLIED,		    /* 17 eol readback flag			*/
    SUPPLIED		    /* 18 pending readback character		*/
};

/*
STRUCT NAME:	hayes
TYPE:		modem
FILE:		IBMPC.C
DESCRIPTION:	Initialization for a Hayes Smartmodem.
*/

struct modem hayes =
{
    _0_SEC_05,		    /*	0 interbyte delay			*/
    OFF,		    /*	1 no modem commands to console		*/
    "AT",		    /*	2 command prefix			*/
    "\015",		    /*	3 command terminator = CR		*/
    "Q0",		    /*	4 sendresult codes			*/
    "",			    /*	5 build all commands here		*/
    _0_SEC_3,		    /*	6 Tsecs to wait after a modem cmd	*/
    4,			    /*	7 number of times to try a command	*/
    _1_SEC_0 + _0_SEC_5,    /*	8 escape guard time (Tsecs)		*/
    FALSE,		    /*	9 not a board modem			*/
    {BAUD1200, BAUD300, BAUD2400, -1},	/* 10 baud rate to use during init  */
    "S2=124",		    /* 11 (S2) online escape byte = '|' 	*/
    SUPPLIED,		    /* 12 baud rate at which modem found	*/
    ON, 		    /* 13 monitor RS-232 signals		*/
    SUPPLIED,		    /* 14 result of modem's product ID cmd      */
    SUPPLIED,		    /* 15 rom checksum				*/
    VIRGIN,		    /* 16 our own code for smodem type		*/
    VIRGIN,		    /* 17 modem successfully configured 	*/
    BAUD_FOUND, 	    /* 18 leave by where modem was found	*/
    SUPPLIED,		    /* 19 maximum baud rate of modem		*/
    "", 		    /* 20 main configuration string		*/
    "T",		    /* 21 tone or pulse 			*/
    "V1",		    /* 22 word result codes			*/
    "E0",		    /* 23 echo commands 			*/
    "S0=0",		    /* 24 (S0)# of rings before autoanswer	*/
    "S6=2",		    /* 25 (S6) dialtone wait (secs)		*/
    "S7=30",		    /* 26 (S7) modem dcd timeout (secs) 	*/
    "M1",		    /* 27 speaker modem 			*/
    "", 		    /* 28 config str for evolutionary cmds	*/
    "X4",		    /* 29 extends result codes for clones	*/
    ",*#()-R",		    /* 30 alpha chars ok in dial string 	*/
    "L1",		    /* 31 speaker volume (1, 2, or 3)		*/
    "&C2",		    /* 32 2400 firmware DCD switch: DCD on	*/
    "&D2",		    /* 33 2400 firnware DTR switch: DTR on	*/
    "Z",		    /* 34 unconfiguration			*/
    "", 		    /* 35 build numeric dial string here	*/
    "D",		    /* 36 dial command				*/
    "", 		    /* 37 dial suffix--e.g., ';' or 'R' 	*/
    "A",		    /* 38 answer phone command			*/
    ON, 		    /* 39 hang up phone 			*/
    "H0",		    /* 40 switch baud rate to incoming call	*/
};

SIO typeAsio =
{
    COMM_SL4,		    /* 0  base address of UART		*/
    DP_OFF,		    /* 1  offset of data port from base	*/
    SP_OFF,		    /* 2  offset of status port from base	*/
    RCV_MASK,		    /* 3  RxRDY mask--defined in U8250.H	*/
    XMIT_MASK,		    /* 4  TBE mask --defined in U8250.H	*/
    inport,		    /* 5  pointer to RAM/port read routine, defined in STDIO.H (SIODEF.H) */
    outport,		    /* 6  pointer to RAM/port write routine defined in STDIO.H (SIODEF.H) */
    s_rcvstat,		    /* 7  pointer to serial receiver status*/
    s_rcv,		    /* 8  pointer to serial fetch routine	*/
    s_xmitstat, 	    /* 9  pointer to serial xmiter status	*/
    s_xmit,		    /* 10 pointer to serial xmit routine	*/
    v_regA,		    /* 11 pointer to virtual register array	*/
    5,			    /* 12 number of rows in virtual array	*/
    &parityA,		    /* 13 parity structure			*/
    &stopsA,		    /* 14 stop bit structure			*/
    &dlenA,		    /* 15 data length structure 		*/
    &baudA,		    /* 16 baud rate structure			*/
    &pin4A,		    /* 17 RS-232 RTS output			*/
    &pin20A,		    /* 18 RS-232 DTR output			*/
    &gp1outA,		    /* 19 RS-232 first user-defined output	*/
    &gp2outA,		    /* 20 RS-232 2nd uset-defined output	*/
    &breakbA,		    /* 21 bresk bit				*/
    &pin5A,		    /* 22 RS-232 CTS input			*/
    &pin6A,		    /* 23 RS-232 DSR input			*/
    &pin8A,		    /* 24 RS-232 DCD input			*/
    &pin22A,		    /* 25 RS-232 RI  input			*/
    NIL,		    /* 26 NO RS-232 gp1 input			*/
    NIL,		    /* 27 NO RS-232 gp2 input			*/
    NIL,		    /* 28 NO transmitter-on structure		*/
    NIL,		    /* 29 NO receiver-on structure		*/
    NIL,		    /* 30 NO transmitter/RTS handshake		*/
    NIL,		    /* 31 NO receiver/DCD handshake		*/
    NIL,		    /* 32 NO clock factor			*/
    SUPPLIED,		    /* 33 decive number supplied during open	*/
    &octlA,		    /* 34 output control			*/
    &ictlA,		    /* 35 input control 			*/
    &hayes,		    /* 36 smartmodem				*/
};

BYTE inport(port)
unsigned port;
{
    return (_inp(port));
}

void outport(port, c)
unsigned port;
char c;
{
    _outp(port, c);
}

/*
FUNCTION NAME:	delay
LEVEL:		0
LIBRARY:	IBMPC.C
DESCTIPTION:	Does nothing for the nember of Tsecs passed in the argument.
*/
#define TIMER_LO    0x46C

void delay(tsecs)
unsigned tsecs;
{
    unsigned tickref;
    for (; tsecs > 0; --tsecs)
    {
	tickref = peekw(TIMER_LO, 0);
	while (tickref == peekw(TIMER_LO, 0));
    }
}

unsigned peekw(addroff, addrseg)
unsigned addroff;
unsigned addrseg;
{
    __asm
    {
	push	ds
	push	si
	mov	si,addroff
	mov	ax,addrseg
	mov	ds,ax
	lodsw
	pop	si
	pop	ds
    }
}

/*
FUNCTION NAME:	s_waitch
LEVEL:		0
LIBRARY:	IBMPC.C
DESCRIPTION:	Waits t secs Tsecs for a serial byte arrive.
OTHER MODULES:	peekw, s_inchar
RETURNS:	int, the byte, if one arrives; otherwise NOT_READY at
		timeout.
COMMENTS:	The system timing interval is defined as a "Tsec", or about 50 msec.
*/

//#define TIMER_LO   0x46C	    /* timer located in segment 0	*/

int s_waitch(siop, tsecs)
register SIO *siop;
unsigned tsecs;
{
    static unsigned tickref;
    tickref = peekw(TIMER_LO, 0);   /* initialize reference count	*/
    while (tsecs > 0)
	{
	if ((*siop->s_rstat)(siop)) /* check UART for byte ready	*/
	    return ((*siop->s_read)(siop)); /* return byte if ready	*/
	if (peekw(TIMER_LO, 0) != tickref)  /* compare new count to old */
	    {
	    tickref = peekw(TIMER_LO, 0);   /* get new reference	*/
	    tsecs--;			    /* ...and reduce timeout	*/
	    }
	}
    return (s_inchar(siop));	    /* timed out--no byte received	*/
}

/*
STRUCT DEF:	_siolist
FILE:		IBMPC.C
DESCRIPTION:	An array of struct _siostat, MAX_PORTS in lengh.
COMMENTS:	The global integer _maxsios contains the maximum number of
		serial devices possible in the system.
*/
int _maxsios = MAX_PORTS;		/* global			*/
struct _siostat _siolist[MAX_PORTS] =	/* IBM has 2 ports		*/
{
    { &typeAsio, CLOSED, TRUE	},	/* COMM1			*/
    { NIL,	 CLOSED, TRUE	},	/* no SIO declared		*/
    { NIL,	 CLOSED, FALSE	},	/* no SIO declared		*/
    { NIL,	 CLOSED, FALSE	},	/* no SIO declared		*/
};

/*
FUNCTION NAME:	isport
LEVEL:		0
LIBRARY:	IBMPC.C
DESCRIPTION:	Verifies that the port number devnum exists.
OTHER MODULES:	peekw: a 2-byte peek function.
RETURNS:	BOOL: TRUE if the device exits.
COMMENTS:	peekw returns an unsigned integer from offset:segment.
*/

#define UART_PTR    0x400	/* pointer to uart table in IBM PC	*/
BOOL isport(devnum)
unsigned devnum;
{
    return (peekw(UART_PTR + (2 * devnum), 0));
}

/*
FUNCTION NAME:	_intron
LEVEL:		0
LIBRARY:	IBMPC.C
DESCRIPTION:	Activate interrupts for the COM port named in the argument
		list.
OTHER MODULES:	setgpo2, __getivec, __setivec, __getcs, __alisr
RETURNS:	void
COMMENTS:	Functions whose names begin with a double underscore are
		assembly-language functions found in module INTRP.ASM. The
		functions __alisrA and __alisrB are the handlers for COM1
		and COM2, respectively. SUPPLIED is a manifest constant in
		SIODEF.H.
*/

typedef unsigned long	INTPTR;     /* typedef for a four-byte pointer	*/

#define I_RXRDY     1		/* values of Interrupt Enable register	*/
