
// This segment also can be appended to STDIO.H.
//#define NULL	0		    /* integer constant 		*/
#define NIL	(void *)0	    /* portable pointer constant	*/
#define NUL	'\0'		    /* ASCII constant			*/
#define ERR	-1
//#define EOF	-1
#define EVER	;;		    /* used in " for (EVER) "		*/
#define FALSE	0
#define TRUE	!FALSE
//#define inport	_inp		    /* port input for UART access	*/
//#define outport _outp		    /* port output (see page 251 of " C Programmer's Guide to Serial Communications "	*/
#define index	strchr
typedef unsigned char BYTE;
typedef unsigned char BOOL;
#define DOS_EOF 0x1A		    /* end of file for text files	*/
#define BLKSIZE 1024
#define MAX_NAME_LEN	80	    /* NAME plus PATH			*/

/********************* The beginning of SIODEF.H. ***********************/

/*
MACRO NAME:	xon, xoff
FILE:		SIODEF.H
DESCRIPTION:	Transmit the XON/XOFF bytes defined in the SIO.
COMMENTS:	The p is a formal parameter for the SIO pointer and is
		replaced with the actual parameter used in the call.
		The parentheses around p are required by most preprocessors.
*/
#define xon(p)	s_putc((p), (p)->s_ictl->xonc)
#define xoff(p) s_putc((p), (p)->s_ictl->xoffc)

#define TIMEOUT     -1		/* timeout flag 			*/
#define IBUFSIZ     256

#define NUMMASKS    5		/* number elements in "setmask" array	*/
//#define NUMBAUDS    17		/* number of baud rates supported	*/
				/* 50 bps up to 38400bps		*/
				/* defined in follow			*/
#define VIRGIN	    -2		/* dummy initializer (must be < -1)	*/
#define SUPPLIED    0		/* dummy initializer (set during init)	*/
#define NA	    0		/* dummy initializer ("not applicable") */
#define ON	    1
#define OFF	    0

#define CLOSED	    FALSE
#define OPEN	    TRUE

#define _0_SEC_05   1		/* 50 ms    */
#define _0_SEC_1    2		/* 1.8	    */
#define _0_SEC_15   3		/* 150 ms   */
#define _0_SEC_2    4		/* 3.6	    */
#define _0_SEC_25   5		/* 250 ms   */
#define _0_SEC_3    6		/* 5.5	    */
#define _0_SEC_4    7		/* 7.3	    */
#define _0_SEC_5    9		/* 9.1	    */
#define _0_SEC_6    12		/* 11.9     */
#define _0_SEC_7    13		/* 12.7     */
#define _0_SEC_8    15		/* 14.6     */
#define _0_SEC_9    16		/* 16.4     */
#define _1_SEC_0    18		/* 18.2     */

#define X1	    0		/* clock factor = 1	*/
#define X16	    1		/* clock factor = 16	*/
#define X32	    2		/* clock factor = 32	*/
#define X64	    3		/* clock factor = 64	*/

#define VIR	    0		/* Manifest constants columns in the	*/
#define USR	    1		/* virtual register array		*/

// Data Format Constants

/* parity constants */
#define NONE	0
#define ODD	1
#define EVEN	2
#define MARK	3
#define SPACE	4

/* stop bits constants	*/
#define STOP1	    0
#define STOP1_5     1
#define STOP2	    2

/* data len constants*/
#define DL5	0
#define DL6	1
#define DL7	2
#define DL8	3

#define NUMBAUDS    17		/* number of baud in divisor array	*/
				/* 50 up to 38400bps			*/
#define BAUD50	    0
#define BAUD75	    1
#define BAUD110     2
#define BAUD134_5   3
#define BAUD150     4
#define BAUD300     5
#define BAUD600     6
#define BAUD1200    7
#define BAUD1800    8
#define BAUD2000    9
#define BAUD2400    10
#define BAUD3600    11
#define BAUD4800    12
#define BAUD7200    13
#define BAUD9600    14
#define BAUD19K2    15
#define BAUD38K4    16

// Return Code Constants

/* Error codes returned by SIO functions    */
#define OR_ARG	    1		/* Argument out of renge    */
#define BAD_FCN     2		/* Function not supported   */
#define BAD_ARG     3		/* Argument not supported   */
#define BAD_PORT    4		/* Illegal device number    */
#define NO_PORT     5		/* Device not installed     */
#define NO_SIO	    6		/* No SIO declared	    */
#define OPEN_NOW    7		/* Device already open	    */
#define NO_CLOSE    8		/* No such device open	    */
#define KEY_CAN     9		/* aborted from keyboard    */
#define NO_FILE     10		/* file not found	    */

struct vregbits_
{
    BYTE	resetmask;	/* AND mask apply to virtual register	*/
    short	setmask[NUMMASKS];  /* array of OR masks for each setting   */
    BYTE	vregnum;	/* virtual register number (row)	*/
    int 	offset; 	/* offset of physical UART register	*/
    BYTE	now;		/* code for current value		*/
    BYTE	start;		/* code to use at initialization	*/
    struct vregbits_	*next;	/* pointer to next vregbits_ structure	*/
};

struct vbaud_
{
    BYTE    vregnuml;		/* virtual array of baud rate lo	*/
    BYTE    vregnumh;		/* virtual array of baud rate hi	*/
    int     offsetlo;		/* offset of lo-order baud rate register*/
    int     offsethi;		/* offset of hi-order baud rate register*/
    int     divisor[NUMBAUDS][2]; /* array of two-byte divisors		*/
    BYTE    now;		/* rank of current divisor		*/
    BYTE    start;		/* rank of value to use at start up	*/
};

/*
STRUCT DEF:	vout232_
LIBRARY:	SIODEF.H
DESCRIPTION:	Structure of RS-232 output data.
*/

struct vout232_
{
    BYTE    resetmask;		/* AND mask apply to vreg		*/
    BYTE    vregnum;		/* number of vitual register		*/
    int     offset;		/* offset of register in uart		*/
    BOOL    now;		/* rank of current setting		*/
    BOOL    start;		/* rank of use at startup		*/
    struct  vout232_	*next;	/* pointer to next struct		*/
};

/*
STRUCT DEF:	vin232_
FILE:		SIODEF.H
DESCRIPTION:	Structure to describe an RS-232 inpur.
COMMENTS:	Contains a pointer to the level-1 status function.
*/

struct vin232_
{
    int     offset;		/* offset of physical register in UART	*/
    BYTE    mask;		/* mask to isolate active bit		*/
    BYTE    now;		/* current value of this input		*/
    BYTE    (*read232)();	/* pointer to level-1 fetch function	*/
};

/*
STRUCT DEF:	soctl_
FILE:		SIODEF.H
DESCRIPTION:	Control structure for "formatted" serial output.
COMMENTS:	Numbering in comments is provided to simplify cross-reference
		with initializers.
*/

struct soctl_
{
    BOOL	rawflg; 	/*  0 perform no processing at all	*/
    short	eolmode;	/*  1 end-of-line formatting		*/
    BOOL	consflg;	/*  2 echo characters to console	*/
    BOOL	wechoflg;	/*  3 interbyte wait for return echo	*/
    unsigned	wecho_dly;	/*  4 Tsecs to wait for return exho	*/
    short	casemode;	/*  5 change to upper or lower case	*/
    short	xxmode; 	/*  6 XON-XOFF control			*/
    BYTE	xoffc;		/*  7 XOFF character			*/
    short	xonc;		/*  8 XON character (-1 == any char)	*/
    BOOL	asciiflg;	/*  9 strip bit 7			*/
    BOOL	cntrlflg;	/* 10 remove control characters		*/
    short	ok_cntrls[32];	/* 11 permitted control characters	*/
    short	numcc;		/* 12 number of controls in ok_cntrls	*/
    BOOL	filterflg;	/* 13 filter-remove characters		*/
    short	filtlist[128][2]; /*14 list of byte to remove or filter	*/
    short	numfc;		/* 15 number of bytes in filter list	*/
    BOOL	ib_dlyflg;	/* 16 interbyte delay in tsecs		*/
    unsigned	ib_dly; 	/* 17 pause between bytes		*/
    BOOL	il_dlyflg;	/* 18 interline pause flag		*/
    BYTE	il_dlyc;	/* 19 char to begin interline pause	*/
    unsigned	il_dly; 	/* 20 interline pause in tsecs		*/
    char	kbortc; 	/* 21 key to break out of loops 	*/
    BYTE	lastc;		/* 22 previous byte output		*/
    BOOL	flushflg;	/* 23 an end of line decision pending	*/
};

/*
STRUCT DEF:	sictl_
FILE:		SIODEF.H
DESCRIPTION:	Control structure for "formated" serial input.
COMMEMTS:	Numering in comments is provided to simplify later
		cross-reference with initializers.
*/

struct sictl_
{
    BOOL	t_outflg;	/*  0 timeout on/off flag		*/
    unsigned	t_out;		/*  1 timeout period			*/
    BOOL	rawflg; 	/*  2 perform no processing at all	*/
    BOOL	ekobakflg;	/*  3 echo all input back to sender	*/
    BOOL	asciiflg;	/*  4 strip bit 7			*/
    short	eolmode;	/*  5 end-of-line formatting		*/
    BOOL	consflg;	/*  6 echo characters to console	*/
    short	casemode;	/*  7 change to upper or lower case	*/
    BYTE	xoffc;		/*  8 XOFFC character			*/
    BYTE	xonc;		/*  9 XONC character			*/
    BOOL	cntrlflg;	/* 10 remove control characters 	*/
    short	ok_cntrls[32];	/* 11 permitted control characters	*/
    short	numcc;		/* 12 number of controls in ok_cntrls	*/
    BOOL	filterflg;	/* 13 filter-remove characters		*/
    int 	filtlist[128][2];   /* 14 list of byte to remove or filter*/
    short	numfc;		/* 15 number of bytes in filter list	*/
    BYTE	lastc;		/* 16 previous byte output		*/
    BOOL	rbakflg;	/* 17 a pending eol decision resolved	*/
    BYTE	rbakc;		/* 18 pending readback character	*/
};

typedef struct
{
    BYTE	*uart_base;	/*  0 base address of UART		*/
    int 	data_off;	/*  1 offset of data port from base	*/
    int 	status_off;	/*  2 offset of status port from base	*/
    unsigned	rcvmask;	/*  3 byte received mask			*/
    unsigned	xmitmask;	/*  4 transmitter holding reg. empty	*/
    BYTE	(*readbyte)();	/*  5 pointer to RAM/port read routine	*/
    void	(*writebyte)(); /*  6 pointer to RAM/port write routine	*/
    BYTE	(*s_rstat)();	/*  7 pointer to serial receiver status	*/
    BYTE	(*s_read)();	/*  8 pointer to serial fetch routine	*/
    BYTE	(*s_xstat)();	/*  9 pointer to serial xmiter status	*/
    void	(*s_send)();	/* 10 pointer to serial xmit routine	*/
    BYTE	(*v_regp)[2];	/* 11 pointer to a 2-dim array of ints	*/
    int 	v_regpsize;	/* 12 length of virtual array		*/
    struct vregbits_	*par;	/* 13 pointer to parity structure	*/
    struct vregbits_	*sb;	/* 14 pointer to stop bit structure	*/
    struct vregbits_	*dl;	/* 15 pointer to data length structure	*/
    struct vbaud_	*br;	/* 16 pointer to baud rate structure	*/
    struct  vout232_	*rts;	/* 17 pointer to RTS structure		*/
    struct  vout232_	*dtr;	/* 18 pointer to DTR structure		*/
    struct  vout232_	*gpo1;	/* 19 pointer to GPOUTPUT1 structure	*/
    struct  vout232_	*gpo2;	/* 20 pointer to GPOUTPUT2 structure	*/
    struct  vout232_	*brkbit;/* 21 pointer to break structure	*/
    struct vin232_  *cts;	/* 22 pointer to CTS structure		*/
    struct vin232_  *dsr;	/* 23 pointer to DSR structure		*/
    struct vin232_  *dcd;	/* 24 pointer to DCD structure		*/
    struct vin232_  *ri;	/* 25 pointer to RI structure		*/
    struct vin232_  *gpi1;	/* 26 pointer to GPINPUT1 structure	*/
    struct vin232_  *gpi2;	/* 27 pointer to GPINPUT2 structure	*/
    struct  vregbits_	*txenable;  /* 28  " xmitter enable structure   */
    struct  vregbits_	*rxenable;  /* 29  " receiver enable structure  */
    struct  vregbits_	*ctshs;     /* 30  " cts handshake on xmitter   */
    struct  vregbits_	*dcdhs;     /* 31  " dcd handshake on receiver  */
    struct  vregbits_	*clkf;	    /* 32  " clock factor structure     */
    short   devnum;		    /* 33 s_opened device number	*/
    struct  soctl_  *s_octl;	/* 34 pointer to output control struct	*/
    struct  sictl_  *s_ictl;	/* 35 pointer to input control struct	*/
    struct  modem   *sm;	/* 36 pointer to modem structure	*/
} SIO;

/*
STRUCT DEF:	_siostat
FILE:		SIODEF.H
DESCRIPTION:	Contains the address of (a pointer to) an SIO, an open/close
		flag, and a flag to indicate whether the SIO supports
		interrupts.
COMMENTS:	One such structure must be declared for each serial port
		supported.
*/

struct _siostat
{
    SIO     *typep;	    /* SIO pointer for this port		*/
    BOOL    openflag;	    /* currently open?				*/
    BOOL    intrptok;	    /* interrupts supported?			*/
};
