

// General XMODEM constant definitions.
#define DBLKSIZ     128 	/* size of data field in packet 	*/
#define HEADROOM    10		/* amount of memory to save from buffer */

#define SOH_TIMOUT  _1_SEC_0	/* timeout interval for SOH		*/
#define ACK_RETRY   30		/* used in loop to wait for ACK 	*/

#define CRCPAKSIZ   133 	/* number of bytes in CRC packet	*/
#define CKSPAKSIZ   132 	/* number of bytes in checksum packet	*/

// Exception code constant definitions.
#define X_ESNDMAX   10	    /* max exceptions during transmission	*/
#define X_ERCVMAX   10	    /* max exceptions druring receiving	*/
#define X_NOSOHMAX  1000    /* max SOH timeout exceptions		*/

#define CONTINUE    0	    /* errcode value that sustains and/rcv loop */
#define BREAK	    1	    /* one of the errcode value that breaks same*/

// Exception code used in both xmit and receive.
#define E_USRCAN    0

// Exception codes used only in x_send.
#define E_FILEMTY   1	    /* file to transmit is empty		*/
#define E_NOACK     2	    /* time out waiting on ACK			*/
#define E_RCVCAN    3	    /* receiver canceled			*/
#define E_BADPAK    4	    /* NAK instead of ACK			*/
#define E_EOF	    5	    /* End of file read 			*/
#define E_LASTACK   6	    /* final ack not acknowledged		*/
#define E_SNDOK     7	    /* successful transmission			*/
#define E_DSKREAD   8	    /* disk read error				*/

// Exception codes used only in x_rcv.
#define E_NOSOH     9	    /* timeout receiving SOH			*/
#define E_BADCKV    10	    /* invalid checkvalue received		*/
#define E_SNDCAN    11	    /* transmitter cancelled transfer		*/
#define E_BADSOH    12	    /* invalid SOH received			*/
#define E_NODATA    13	    /* timeout receiving data in block		*/
#define E_PAKNUM    14	    /* invalid block number received		*/
#define E_SNDACK    15	    /* dender missed last ack			*/
#define E_PAKSEQ    16	    /* FATAL: packet out of sequence		*/
#define E_DSKWRITE  17	    /* disk write error 			*/
#define E_RCVOK     18	    /* successful reception			*/

/*
STRUCT DEF:	sndpacket
FILE:		XMOD.H
DESCRIPTION:	Description of an XMODEM packet for transmission.
*/

struct sndpacket
{
    BYTE    even;		/* makes sndpacket.ckval is continued by sndpacket.data in memory address*/
    BYTE    soh;		/* start of header			*/
    BYTE    pnum1;		/* packet number			*/
    BYTE    pnum2;		/* 1's comp packet number               */
    BYTE    data[DBLKSIZ];	/* data block (127 bytes with 1 end-byte*/
    unsigned short  ckval;	/* block-check character		*/
};

/*
STRUCT DEF:	rcvpacket
FILE:		XMOD.H
DESCRIPTION:	Structure to describe a packet during reception.
COMMENTS:	All members are declared int. There is no member for the
		SOH.
*/

struct rcvpacket
{
    unsigned short pnum1;		/* packet number		*/
    unsigned short pnum2;		/* 1's comp packet number       */
    unsigned short data[DBLKSIZ];	/* 128-byte data block		*/
    unsigned short ckvhi;		/* high byte of checkval	*/
    unsigned short chvlo;		/* low byte of checkval 	*/
};
